#!/usr/bin/env ruby

require 'book'

class Scanner
  
  def set_word(word, pos)
    return if word.empty?
    if @word_set[word].nil?
       @word_set[word] = {}
    end
    path_set = @word_set[word]
    if path_set[@path_index].nil?
       path_set[@path_index] = []
    end
    path_set[@path_index].push pos
  end
  
  def initialize(word_set={})
    @word_set = word_set
  end
  
  attr :word_set
  attr :path_index, true
  
  def scan_word(file)
    offset = 0
    while line = file.gets
      word = ''
      pos = 0
      line.scan(/./m) do |char|
        match_pos = offset + $~.begin(0)
        if char.size == 1
           if %r[[\w\d_/+-]] !~ char
             set_word(word, pos)
             word = ''
             next
           end
           pos = match_pos if word.empty?
           word+= char.downcase
           next
        else
           set_word(word, pos)
           pos = match_pos
           word = char
        end
        set_word(word, pos)
        word = ''
      end
      set_word(word, pos)
      word = ''
      offset+= line.size
    end
  end

end

SIZE_LIMIT = 1024*1024

book = Book.open

if book.page_count_limit == 0
  dir = ARGV.shift || ''
  book.page_reset
  size = 0
  Dir.glob(dir + '**/*.txt') do |path|
    size+= File.stat(path).size
    if size >= SIZE_LIMIT
      size = 0
      book.page_new
    end
    book.page_put_path(path)
  end
  book.close
end

book.each do
  path_list = book.path_list_update
  if path_list.empty?
    puts "page #{book.page_count} not updated."
    next
  end
  scanner = Scanner.new
  path_list.each do |scanner.path_index, path|
    puts path
    open(path, 'rb') do |file|
      scanner.scan_word(file)
    end
  end
  wpos = book.word_set_save(scanner.word_set)
  book.wpos_save(wpos)
  puts "page #{book.page_count} updated."
end
