#!/usr/bin/env ruby

module DojoCode

module_function

  module SJIS
  module_function
    def hicode?(ch)
      (0x81..0x9f) === ch || (0xe0..0xfc) === ch
    end
    def locode?(ch)
      (0x40..0x7e) === ch || (0x80..0xfc) === ch
    end
  end
  
  module EUC
  module_function
    def hicode?(ch)
      (0xa1..0xfe) === ch || 0x8e == ch
    end
    def locode?(ch)
      (0xa0..0xfe) === ch
    end
  end
  
  module NONE
  module_function
    def hicode?(ch)
      (0x80..0xff) === ch
    end
    def locode?(ch)
      (0x80..0xff) === ch
    end
  end
  
  def hicode?(ch)
    @@kcode.hicode?(ch)
  end
  
  def locode?(ch)
    @@kcode.locode?(ch)
  end
  
  def conform_wbtype?(sample, wbtype)
    (sample.size-1).downto(1) do |n|
      case wbtype
      when 0
        return true  unless hicode?(sample[n-1])
        return false
      when 1
        return true  unless hicode?(sample[n-1])
        return false unless locode?(sample[n-1])
        wbtype = 2
      when 2
        return false unless hicode?(sample[n-1])
        wbtype = 1
      end
    end
    nil
  end
  
  def end_wide?(sample)
    n = sample.size-1
    hicode?(sample[n]) || (locode?(sample[n]) && hicode?(sample[n-1]))
  end
  
  def end_type(sample)
    return 0 unless end_wide?(sample)
    t1 = conform_wbtype?(sample, 1)
    t2 = conform_wbtype?(sample, 2)
    return 1 if  t1 && !t2
    return 2 if !t1 &&  t2
    nil
  end
  
  SAMPLE_LENGTH = 16
  
  def wbtype(file, position)
    file.pos = [position, 0].max
    return if file.eof?
    
    offset = 1
    pos = SAMPLE_LENGTH
    begin
      if pos <= 0
        ret = position%2 + 1
        break
      end
      offset -= SAMPLE_LENGTH
      pos = position + offset
      if pos <= 0
        file.pos = 0
        sample = file.read(SAMPLE_LENGTH + pos)
      else
        file.pos = pos
        sample = file.read(SAMPLE_LENGTH)
      end
    end until ret = end_type(sample)
    if offset != 1-SAMPLE_LENGTH && ret == 0
      ret = 2
    end
    ret
  end
  
  RE_SJIS = /([\201-\237\340-\374][\100-\176\200-\374])+/
  RE_EUC  = /([\241-\376][\241-\376])+/
  
  def kcode(file)
    s = 0
    e = 0
    pos = file.pos
    file.pos = 0
    while file.gets
      scan(RE_SJIS){|i| s += i.size }
      scan(RE_EUC ){|i| e += i.size }
    end
    file.pos = pos
    [EUC, NONE, SJIS][(s<=>e)+1]
  end
  
  def set_kcode(kcode)
    @@kcode = kcode
  end
  
end # of module DojoCode

if __FILE__ == $0
  open(ARGV.shift || 'readme.txt', 'rb') do |f|
  p kcode = DojoCode.kcode(f)
    DojoCode.set_kcode(kcode)
    for i in 0...16
      print ' ', DojoCode.wbtype(f, i)
    end
  end
end
